package sun.misc;

import java.util.HashSet;
import java.security.PermissionCollection;
import java.security.PrivilegedAction;
import java.security.CodeSource;
import java.security.cert.Certificate;
import java.security.ProtectionDomain;
import java.security.AccessControlContext;
import java.net.URI;
import java.nio.file.Paths;
import java.net.URISyntaxException;
import java.util.Vector;
import java.util.StringTokenizer;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.net.URLClassLoader;
import java.net.MalformedURLException;
import sun.net.www.ParseUtil;
import java.net.URL;
import java.io.File;
import java.io.IOException;
import java.net.URLStreamHandler;
import java.net.URLStreamHandlerFactory;

public class Launcher
{
    private static URLStreamHandlerFactory factory;
    private static Launcher launcher;
    private static String bootClassPath;
    private ClassLoader loader;
    private static URLStreamHandler fileHandler;
    
    public static Launcher getLauncher() {
        return Launcher.launcher;
    }
    
    public Launcher() {
        Launcher.ExtClassLoader extClassLoader;
        try {
            extClassLoader = Launcher.ExtClassLoader.getExtClassLoader();
        }
        catch (IOException cause) {
            throw new InternalError("Could not create extension class loader", cause);
        }
        try {
            this.loader = Launcher.AppClassLoader.getAppClassLoader(extClassLoader);
        }
        catch (IOException cause2) {
            throw new InternalError("Could not create application class loader", cause2);
        }
        Thread.currentThread().setContextClassLoader(this.loader);
        String property = System.getProperty("java.security.manager");
        if (property != null) {
            SecurityManager securityManager = null;
            if ("".equals(property) || "default".equals(property)) {
                securityManager = new SecurityManager();
            }
            else {
                try {
                    securityManager = (SecurityManager)this.loader.loadClass(property).newInstance();
                }
                catch (IllegalAccessException ex) {}
                catch (InstantiationException ex2) {}
                catch (ClassNotFoundException ex3) {}
                catch (ClassCastException ex4) {}
            }
            if (securityManager != null) {
                System.setSecurityManager(securityManager);
            }
            else {
                throw new InternalError("Could not create SecurityManager: " + property);
            }
        }
    }
    
    public ClassLoader getClassLoader() {
        return this.loader;
    }
    
    public static URLClassPath getBootstrapClassPath() {
        return Launcher.BootClassPathHolder.bcp;
    }
    
    private static URL[] pathToURLs(File[] array) {
        URL[] array2 = new URL[array.length];
        for (int i = 0; i < array.length; ++i) {
            array2[i] = getFileURL(array[i]);
        }
        return array2;
    }
    
    private static File[] getClassPath(String s) {
        if (s != null) {
            int n = 0;
            int n2 = 1;
            int index;
            for (int fromIndex = 0; (index = s.indexOf(File.pathSeparator, fromIndex)) != -1; fromIndex = index + 1) {
                ++n2;
            }
            File[] array = new File[n2];
            int beginIndex;
            int index2;
            for (beginIndex = 0; (index2 = s.indexOf(File.pathSeparator, beginIndex)) != -1; beginIndex = index2 + 1) {
                if (index2 - beginIndex > 0) {
                    array[n++] = new File(s.substring(beginIndex, index2));
                }
                else {
                    array[n++] = new File(".");
                }
            }
            if (beginIndex < s.length()) {
                array[n++] = new File(s.substring(beginIndex));
            }
            else {
                array[n++] = new File(".");
            }
            if (n != n2) {
                File[] array2 = new File[n];
                System.arraycopy(array, 0, array2, 0, n);
                array = array2;
            }
            return array;
        }
        else {
            return new File[0];
        }
    }
    
    static URL getFileURL(File canonicalFile) {
        try {
            canonicalFile = canonicalFile.getCanonicalFile();
        }
        catch (IOException ex) {}
        try {
            return ParseUtil.fileToEncodedURL(canonicalFile);
        }
        catch (MalformedURLException cause) {
            throw new InternalError(cause);
        }
    }
    
    static File[] access$300(String s) {
        return getClassPath(s);
    }
    
    static URL[] access$400(File[] array) {
        return pathToURLs(array);
    }
    
    static {
        Launcher.factory = new Launcher.Factory();
        Launcher.launcher = new Launcher();
        Launcher.bootClassPath = System.getProperty("sun.boot.class.path");
    }
    
    static class ExtClassLoader extends URLClassLoader
    {
        private static volatile Launcher.ExtClassLoader instance;
        
        public static Launcher.ExtClassLoader getExtClassLoader() throws IOException {
            if (Launcher.ExtClassLoader.instance == null) {
                synchronized (Launcher.ExtClassLoader.class) {
                    if (Launcher.ExtClassLoader.instance == null) {
                        Launcher.ExtClassLoader.instance = createExtClassLoader();
                    }
                }
            }
            return Launcher.ExtClassLoader.instance;
        }
        
        private static Launcher.ExtClassLoader createExtClassLoader() throws IOException {
            try {
                return (Launcher.ExtClassLoader)AccessController.doPrivileged(new PrivilegedExceptionAction<Launcher.ExtClassLoader>() {
                    @Override
                    public Launcher.ExtClassLoader run() throws IOException {
                        File[] access$100 = Launcher.ExtClassLoader.access$100();
                        for (int length = access$100.length, i = 0; i < length; ++i) {
                            MetaIndex.registerDirectory(access$100[i]);
                        }
                        return new ExtClassLoader(access$100);
                    }
                });
            }
            catch (PrivilegedActionException ex) {
                throw (IOException)ex.getException();
            }
        }
        
        void addExtURL(URL url) {
            super.addURL(url);
        }
        
        public ExtClassLoader(File[] array) throws IOException {
            super(getExtURLs(array), null, Launcher.factory);
            SharedSecrets.getJavaNetAccess().getURLClassPath(this).initLookupCache(this);
        }
        
        private static File[] getExtDirs() {
            String property = System.getProperty("java.ext.dirs");
            if (property != null) {
                StringTokenizer stringTokenizer = new StringTokenizer(property, File.pathSeparator);
                int countTokens = stringTokenizer.countTokens();
                File[] array = new File[countTokens];
                for (int i = 0; i < countTokens; ++i) {
                    array[i] = new File(stringTokenizer.nextToken());
                }
                return array;
            }
            return new File[0];
        }
        
        private static URL[] getExtURLs(File[] array) throws IOException {
            Vector<URL> vector = new Vector();
            for (int i = 0; i < array.length; ++i) {
                String[] list = array[i].list();
                if (list != null) {
                    for (int j = 0; j < list.length; ++j) {
                        if (!list[j].equals("meta-index")) {
                            vector.add(Launcher.getFileURL(new File(array[i], list[j])));
                        }
                    }
                }
            }
            URL[] anArray = new URL[vector.size()];
            vector.copyInto(anArray);
            return anArray;
        }
        
        public String findLibrary(String mapLibraryName) {
            mapLibraryName = System.mapLibraryName(mapLibraryName);
            URL[] urLs = super.getURLs();
            File obj = null;
            for (int i = 0; i < urLs.length; ++i) {
                URI uri;
                try {
                    uri = urLs[i].toURI();
                }
                catch (URISyntaxException ex) {
                    continue;
                }
                File file = (uri.getAuthority() == null) ? new File(uri).getParentFile() : Paths.get(uri).toFile().getParentFile();
                if (file != null && !file.equals(obj)) {
                    String savedProperty = VM.getSavedProperty("os.arch");
                    if (savedProperty != null) {
                        File o = new File(new File(file, savedProperty), mapLibraryName);
                        if (o.exists()) {
                            return o.getAbsolutePath();
                        }
                    }
                    File o2 = new File(file, mapLibraryName);
                    if (o2.exists()) {
                        return o2.getAbsolutePath();
                    }
                }
                obj = file;
            }
            return null;
        }
        
        private static AccessControlContext getContext(File[] array) throws IOException {
            PathPermissions permissions = new PathPermissions(array);
            return new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(new CodeSource(permissions.getCodeBase(), (Certificate[])null), permissions) });
        }
        
        static File[] access$100() {
            return getExtDirs();
        }
        
        static {
            ClassLoader.registerAsParallelCapable();
            Launcher.ExtClassLoader.instance = null;
        }
    }
    
    static class AppClassLoader extends URLClassLoader
    {
        final URLClassPath ucp;
        
        public static ClassLoader getAppClassLoader(ClassLoader classLoader) throws IOException {
            String property = System.getProperty("java.class.path");
            return (ClassLoader)AccessController.doPrivileged(new PrivilegedAction<Launcher.AppClassLoader>() {
                
                @Override
                public Launcher.AppClassLoader run() {
                    return new Launcher.AppClassLoader((property == null) ? new URL[0] : Launcher.access$400(path), classLoader);
                }
            });
        }
        
        AppClassLoader(URL[] urls, ClassLoader parent) {
            super(urls, parent, Launcher.factory);
            (this.ucp = SharedSecrets.getJavaNetAccess().getURLClassPath(this)).initLookupCache(this);
        }
        
        public Class<?> loadClass(String s, boolean resolve) throws ClassNotFoundException {
            int lastIndex = s.lastIndexOf(46);
            if (lastIndex != -1) {
                SecurityManager securityManager = System.getSecurityManager();
                if (securityManager != null) {
                    securityManager.checkPackageAccess(s.substring(0, lastIndex));
                }
            }
            if (this.ucp.knownToNotExist(s)) {
                Class<Object> loadedClass = this.findLoadedClass(s);
                if (loadedClass != null) {
                    if (resolve) {
                        this.resolveClass(loadedClass);
                        return loadedClass;
                    }
                    return loadedClass;
                }
                else {
                    throw new ClassNotFoundException(s);
                }
            }
            else {
                return super.loadClass(s, resolve);
            }
        }
        
        @Override
        protected PermissionCollection getPermissions(CodeSource codesource) {
            PermissionCollection permissions = super.getPermissions(codesource);
            permissions.add(new RuntimePermission("exitVM"));
            return permissions;
        }
        
        private void appendToClassPathForInstrumentation(String pathname) {
            assert !(!Thread.holdsLock(this));
            super.addURL(Launcher.getFileURL(new File(pathname)));
            return;
        }
        
        private static AccessControlContext getContext(File[] array) throws MalformedURLException {
            PathPermissions permissions = new PathPermissions(array);
            return new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(new CodeSource(permissions.getCodeBase(), (Certificate[])null), permissions) });
        }
        
        static {
            ClassLoader.registerAsParallelCapable();
        }
    }
    
    private static class BootClassPathHolder
    {
        static final URLClassPath bcp;
        
        static {
            URL[] array;
            if (Launcher.bootClassPath != null) {
                array = (URL[])AccessController.doPrivileged(new PrivilegedAction<URL[]>() {
                    @Override
                    public URL[] run() {
                        File[] access$300 = Launcher.access$300(Launcher.bootClassPath);
                        int length = access$300.length;
                        HashSet<File> set = new HashSet();
                        for (File parentFile : access$300) {
                            if (!parentFile.isDirectory()) {
                                parentFile = parentFile.getParentFile();
                            }
                            if (parentFile != null && set.add(parentFile)) {
                                MetaIndex.registerDirectory(parentFile);
                            }
                        }
                        return Launcher.access$400(access$300);
                    }
                });
            }
            else {
                array = new URL[0];
            }
            (bcp = new URLClassPath(array, Launcher.factory, null)).initLookupCache(null);
        }
    }
    
    private static class Factory implements URLStreamHandlerFactory
    {
        private static String PREFIX;
        
        @Override
        public URLStreamHandler createURLStreamHandler(String s) {
            String string = Launcher.Factory.PREFIX + "." + s + ".Handler";
            try {
                return (URLStreamHandler)Class.forName(string).newInstance();
            }
            catch (ReflectiveOperationException cause) {
                throw new InternalError("could not load " + s + "system protocol handler", cause);
            }
        }
        
        static {
            Launcher.Factory.PREFIX = "sun.net.www.protocol";
        }
    }
}
